/*
 * *********************************************************
 * Copyright (c) 2009 - 2010, DHBW Mannheim - Tigers Mannheim
 * Project: TIGERS - Sumatra
 * Date: 23.08.2010
 * Author(s): AndreR
 * *********************************************************
 */
package edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.botmanager.bots.communication;

import com.sleepycat.persist.model.Persistent;

import edu.dhbw.mannheim.tigers.sumatra.util.clock.SumatraClock;


/**
 * Communication statistics packet.
 * 
 * @author AndreR
 */
@Persistent
public class Statistics
{
	/** */
	public int		payload	= 0;
	/** */
	public int		raw		= 0;
	/** */
	public int		packets	= 0;
	private long	lastReset;
	
	
	/**
	 */
	public Statistics()
	{
		lastReset = SumatraClock.nanoTime();
	}
	
	
	/**
	 * @param org
	 */
	public Statistics(final Statistics org)
	{
		payload = org.payload;
		raw = org.raw;
		packets = org.packets;
		lastReset = org.lastReset;
	}
	
	
	/**
 */
	public void reset()
	{
		payload = 0;
		raw = 0;
		packets = 0;
		
		lastReset = SumatraClock.nanoTime();
	}
	
	
	/**
	 * @return
	 */
	public long getLastResetTimestamp()
	{
		return lastReset;
	}
	
	
	/**
	 * @param stat
	 * @return
	 */
	public Statistics substract(final Statistics stat)
	{
		final Statistics ret = new Statistics();
		ret.payload = payload - stat.payload;
		ret.raw = raw - stat.raw;
		ret.packets = packets - stat.packets;
		
		return ret;
	}
	
	
	/**
	 * @param stat
	 * @return
	 */
	public Statistics add(final Statistics stat)
	{
		final Statistics ret = new Statistics();
		ret.payload = payload + stat.payload;
		ret.raw = raw + stat.raw;
		ret.packets = packets + stat.packets;
		
		return ret;
	}
	
	
	/**
	 * @return
	 */
	public float getOverheadPercentage()
	{
		if (raw == 0)
		{
			return 0;
		}
		
		return (1.0f - (((float) payload) / ((float) raw)));
	}
	
	
	/**
	 * @param passedTime
	 * @return
	 */
	public float getLoadPercentage(final float passedTime)
	{
		if (passedTime == 0)
		{
			return 0;
		}
		
		return ((raw) / (28800.0f * passedTime));
	}
	
	
	/**
	 * @return
	 */
	public float getLoadPercentageWithLastReset()
	{
		return getLoadPercentage((SumatraClock.nanoTime() - lastReset) / 1000000000.0f);
	}
}
